/*
 * A C# program that tests painting by
 * creating a custom component with its own renderer.
 *
 * Ben Bederson, March 31, 2005
 * University of Maryland
 */

using System;
using System.Net;
using System.IO;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;

public class BasicGraphics : System.Windows.Forms.Form {
	static public void Main() {
		Application.Run(new BasicGraphics());
	}

	public BasicGraphics() {
		Size = new Size(800, 800);
		StartPosition = FormStartPosition.CenterScreen;

		BasicControl c1 = new BasicControl();
		c1.Bounds = new Rectangle(0, 0, 400, 400);

		StrokeControl c2 = new StrokeControl();
		c2.Bounds = new Rectangle(400, 0, 400, 400);

		RegionControl c3 = new RegionControl();
		c3.Bounds = new Rectangle(0, 400, 400, 400);

		ImageControl c4 = new ImageControl();
		c4.Bounds = new Rectangle(400, 400, 400, 400);
		c4.Image = new Bitmap(FetchImage("http://www.cs.umd.edu/~bederson/images/bederson.gif"));

		Controls.Add(c1);
		Controls.Add(c2);
		Controls.Add(c3);
		Controls.Add(c4);
	}

	public Image FetchImage(string url) {
		HttpWebRequest webRequest = (HttpWebRequest)WebRequest.Create(url);
		WebResponse webResponse = webRequest.GetResponse();
		Stream stream = webResponse.GetResponseStream();
		return Bitmap.FromStream(stream);
	}
}

class BasicControl : Control {
	static Brush blackBrush = new SolidBrush(Color.Black);
	static Font font = new Font("Arial", 15);

	protected override void OnPaint(PaintEventArgs e) {
		Graphics g = e.Graphics;
		BackColor = Color.White;
		g.DrawString("Hello World!", font, blackBrush, 100, 100);
	}
}

class StrokeControl : Control {
	static Pen redPen = new Pen(Color.Red);

	protected override void OnPaint(PaintEventArgs e) {
		Graphics g = e.Graphics;
		BackColor = Color.White;

		redPen.LineJoin = LineJoin.Round;
		redPen.StartCap = LineCap.Round;
		redPen.EndCap = LineCap.Round;
		redPen.Width = 10;
		g.DrawRectangle(redPen, 10, 10, 100, 100);
		g.DrawLine(redPen, 50, 50, 200, 100);

		redPen.StartCap = LineCap.Flat;
		redPen.EndCap = LineCap.Flat;
		g.SmoothingMode = SmoothingMode.AntiAlias;
		g.DrawLine(redPen, 200, 50, 50, 100);
	}
}

class RegionControl : Control {
	protected override void OnPaint(PaintEventArgs e) {
		Graphics g = e.Graphics;
		BackColor = Color.White;

		g.FillRectangle(Brushes.Blue, 10, 10, 100, 100);

		Brush brush = new LinearGradientBrush(new Point(0, 0), new Point(100, 100), Color.Red, Color.Yellow);
		g.FillRectangle(brush, 150, 10, 100, 100);
	}
}

class ImageControl : Control {
	private Image image;

	public Image Image {
		get { return image; }
		set { 
			image = value;
			Invalidate();
		}
	}

	protected override void OnPaint(PaintEventArgs e) {
		Graphics g = e.Graphics;
		BackColor = Color.White;

		g.DrawImage(image, 0, 0);
	}
}